from django.core import mail
from django.utils import timezone
from django.utils.dateparse import parse_datetime

import pytest
from freezegun import freeze_time
from model_bakery import baker

from castellum.appointments.helpers import send_appointment_notifications
from castellum.appointments.models import Appointment


def dt(s):
    return timezone.make_aware(parse_datetime(s))


@pytest.mark.parametrize('start', [
    '2017-01-01 12:00',
    '2017-01-02 12:00',
])
def test_appointment_notifications(user, member, participation, start):
    appointment = baker.make(Appointment, participation=participation, start=dt(start))
    with freeze_time('2017-01-01'):
        send_appointment_notifications(
            participation,
            [(dt(start), appointment, ['test@example.com'])],
            'http://example.com',
            user=user,
        )
    assert len(mail.outbox) == 1


@pytest.mark.parametrize('start', [
    '2016-01-01 12:00',
    '2016-12-31 12:00',
    '2017-01-03 12:00',
    '2018-01-01 12:00',
])
def test_appointment_notifications_no_relevant_change(
    user, member, participation, start
):
    appointment = baker.make(Appointment, participation=participation, start=dt(start))
    with freeze_time('2017-01-01'):
        send_appointment_notifications(
            participation,
            [(
                dt(start),
                None,
                Appointment.get_conductor_mails(
                    appointment=appointment, participation=participation
                )
            )],
            'http://example.com',
            user=user,
        )
    assert len(mail.outbox) == 0


def test_appointment_notifications_no_change(user, member, participation):
    send_appointment_notifications(
        participation,
        [],
        'http://example.com',
        user=user,
    )
    assert len(mail.outbox) == 0


def test_appointment_notifications_no_members(user, participation):
    appointment = baker.make(Appointment, start=dt('2017-01-01 13:00'), participation=participation)
    with freeze_time('2017-01-01'):
        send_appointment_notifications(
            participation,
            [(
                dt('2017-01-01 12:00'),
                appointment,
                Appointment.get_conductor_mails(
                    appointment=appointment, participation=participation
                )
            )],
            'http://example.com',
            user=user,
        )
    assert len(mail.outbox) == 0


def test_appointment_notifications_weekend(user, member, participation):
    appointment = baker.make(Appointment, start=dt('2017-01-09 13:00'), participation=participation)
    with freeze_time('2017-01-06'):
        send_appointment_notifications(
            participation,
            [(
                dt('2017-01-09 12:00'),
                appointment,
                Appointment.get_conductor_mails(
                    appointment=appointment, participation=participation
                )
            )],  # friday to monday
            'http://example.com',
            user=user,
        )
    assert len(mail.outbox) == 1


@pytest.mark.parametrize('start', [
    '2017-01-01 12:00',
    '2017-01-02 12:00',
])
def test_appointment_notifications_cc(user, conductor, participation, start):
    with freeze_time('2017-01-01'):
        send_appointment_notifications(
            participation,
            [(
                dt(start),
                None,
                Appointment.get_conductor_mails(participation=participation, appointment=None),
            )],
            'http://example.com',
            user=user,
        )
    assert len(mail.outbox) == 1
    assert mail.outbox[0].cc == [user.email]


@pytest.mark.parametrize('start', [
    '2017-01-01 12:00',
    '2017-01-02 12:00',
])
def test_no_appointment_notifications_if_only_conductor(conductor, participation, start):
    with freeze_time('2017-01-01'):
        send_appointment_notifications(
            participation,
            [(
                dt(start),
                None,
                Appointment.get_conductor_mails(participation=participation, appointment=None),
            )],
            'http://example.com',
            user=conductor,
        )
    assert len(mail.outbox) == 0
